@@ -18,10 +18,17 @@ X<Lets Encrypt> X<Let's Encrypt> X<letsencrypt>
     use parent qw( Net::ACME2 );
 
     use constant {
-        HOST => 'acme.someca.net',
         DIRECTORY_PATH => '/acme-directory',
     };
 
+    # %opts are the parameters given to new().
+    sub HOST {
+        my ($class, %opts) = @_;
+
+        # You can make this depend on the %opts if you want.
+        return 'acme.someca.net';
+    }
+
     package main;
 
     my $acme = SomeCA::ACME->new(
@@ -54,7 +61,7 @@ X<Lets Encrypt> X<Let's Encrypt> X<letsencrypt>
 
     $acme->accept_challenge($challenge);
 
-    sleep 1 while !$acme->poll_authorization($authz);
+    sleep 1 while 'valid' ne $acme->poll_authorization($authz);
 
     # ... Make a key and CSR for *.example.com
 
@@ -69,7 +76,7 @@ X<Lets Encrypt> X<Let's Encrypt> X<letsencrypt>
 
     # ... Download your certificate! :)
 
-See F</examples> in the distribution for a more fleshed-out example.
+See F</examples> in the distribution for more fleshed-out examples.
 
 To use L<Let’s Encrypt|http://letsencrypt.org>, see
 L<Net::ACME2::LetsEncrypt>.
@@ -85,28 +92,32 @@ Net::ACME2 derives from L<Net::ACME>, which implements the
 (significantly different) earlier draft of that protocol as initially
 deployed by L<Let’s Encrypt|http://letsencrypt.org>.
 
-Net::ACME2 is pure Perl, and all of its dependencies are either pure Perl
-or core modules.
-
 =head1 STATUS
 
-This is an alpha-grade implementation. While the underlying protocol
+This is a beta-grade implementation. While the underlying protocol
 is L<in use for production|https://community.letsencrypt.org/t/acme-v2-production-environment-wildcards/55578>,
 it’s still not finalized; consequently, this distribution remains
-subject to change.
+subject to change. It is expected that any further breaking changes
+will be small, but you still B<MUST> check the changelog before upgrading!
 
 =head1 FEATURES
 
 =over
 
-=item * This is a pure-Perl solution, all of whose dependencies are either
-core modules or pure-Perl themselves. Net::ACME2 will run anywhere that Perl
-runs. :)
-
 =item * Support for both ECDSA and RSA encrytion.
 
+=item * Support for http-01, dns-01, and tls-alpn-01 challenges.
+
 =item * Comprehensive error handling with typed, L<X::Tiny>-based exceptions.
 
+=item * This is a pure-Perl solution. Most of its dependencies are
+either core modules or pure Perl themselves. XS is necessary to
+communicate with the ACME server via TLS; however, most Perl installations
+already include the necessary logic (i.e., L<Net::SSLeay>) for TLS.
+
+In short, Net::ACME2 will run anywhere that Perl can speak TLS, which is
+I<almost> everywhere that Perl runs.
+
 =back
 
 =head1 ERROR HANDLING
@@ -114,30 +125,35 @@ runs. :)
 All thrown exceptions are instances of L<Net::ACME2::X::Base>.
 Specific error classes aren’t yet defined.
 
+=head1 SPEED
+
+If you notice speed problems, check to see if your L<Math::BigInt>
+installation can be made faster.
+
 =cut
 
-use Crypt::Format ();
+use Crypt::Format;
+use Crypt::Perl::PK;
 use MIME::Base64 ();
 
-use Net::ACME2::Constants ();
-use Net::ACME2::HTTP ();
-use Net::ACME2::Order ();
-use Net::ACME2::Authorization ();
+use Net::ACME2::HTTP;
+use Net::ACME2::Order;
+use Net::ACME2::Authorization;
 
-our $VERSION;
-*VERSION = *Net::ACME2::Constants::VERSION;
+our $VERSION = '0.21';
 
 use constant {
-    JWS_FORMAT => undef,
-
     _JWK_THUMBPRINT_DIGEST => 'sha256',
 };
 
+# accessed from test
 use constant newAccount_booleans => qw(
     termsOfServiceAgreed
     onlyReturnExisting
 );
 
+# the list of methods that need a “jwk” in their JWS Protected Header
+# (cf. section 6.2 of the spec)
 use constant FULL_JWT_METHODS => qw(
     newAccount
     revokeCert
@@ -153,7 +169,7 @@ interactions with the ACME server. %OPTS is:
 =over
 
 =item * C<key> - Required. The private key to associate with the ACME2
-user. PEM or DER format.
+user. Anything that C<Crypt::Perl::PK::parse_key()> can parse is acceptable.
 
 =item * C<key_id> - Optional. As returned by C<key_id()>.
 Saves a round-trip to the ACME2 server, so you should give this
@@ -171,10 +187,10 @@ emptor.
 sub new {
     my ( $class, %opts ) = @_;
 
-    die 'Need “key”!' if !$opts{'key'};
+    _die_generic('Need “key”!') if !$opts{'key'};
 
     my $self = {
-        _host => $class->HOST(),
+        _host => $class->HOST(%opts),
         _key  => $opts{'key'},
         _key_id => $opts{'key_id'},
         _directory => $opts{'directory'},
@@ -187,7 +203,7 @@ sub new {
     return $self;
 }
 
-=head2 I<OBJ>->key_id()
+=head2 $id = I<OBJ>->key_id()
 
 Returns the object’s cached key ID, either as given at instantiation
 or as fetched in C<create_new_account()>.
@@ -200,9 +216,8 @@ sub key_id {
     return $self->{'_key_id'};
 }
 
-=head2 I<CLASS_OR_OBJ>->get_terms_of_service()
+=head2 $url = I<OBJ>->get_terms_of_service()
 
-Callable as either an instance method or a class method.
 Returns the URL for the terms of service.
 
 B<NOTE:> For L<Let’s Encrypt|http://letsencrypt.org> you can
@@ -221,13 +236,16 @@ sub get_terms_of_service {
     }
 
     my $dir = $self->_get_directory();
-    my $url = $self->_get_directory()->{'meta'} or die 'No “meta” in directory!';
-    $url = $url->{'termsOfService'} or die 'No “terms-of-service” in directory metadata!';
+
+    # Exceptions here indicate an ACME violation and should be
+    # practically nonexistent.
+    my $url = $dir->{'meta'} or _die_generic('No “meta” in directory!');
+    $url = $url->{'termsOfService'} or _die_generic('No “termsOfService” in directory metadata!');
 
     return $url;
 }
 
-=head2 I<OBJ>->create_new_account( %OPTS )
+=head2 $created_yn = I<OBJ>->create_new_account( %OPTS )
 
 Creates a new account using the ACME2 object’s key and the passed
 %OPTS, which are as described in the ACME2 spec (cf. C<newAccount>).
@@ -241,7 +259,10 @@ or 0 if the account already existed.
 sub create_new_account {
     my ($self, %opts) = @_;
 
-    $opts{$_} &&= JSON::true() for newAccount_booleans();
+    for my $name (newAccount_booleans()) {
+        next if !exists $opts{$name};
+        ($opts{$name} &&= JSON::true()) ||= JSON::false();
+    }
 
     my $resp = $self->_post(
         'newAccount',
@@ -259,7 +280,10 @@ sub create_new_account {
     my $struct = $resp->content_struct();
 
     if ($struct) {
-        $struct->{$_} = !!$struct->{$_} for newAccount_booleans();
+        for my $name (newAccount_booleans()) {
+            next if !exists $struct->{$name};
+            ($struct->{$name} &&= 1) ||= 0;
+        }
     }
 
     return 1;
@@ -278,7 +302,7 @@ sub create_new_account {
 #    return $set;
 #}
 
-=head2 I<OBJ>->create_new_order( %OPTS )
+=head2 $order = I<OBJ>->create_new_order( %OPTS )
 
 Returns a L<Net::ACME2::Order> object. %OPTS is as described in the
 ACME spec (cf. C<newOrder>). Boolean values may be given as simple
@@ -301,12 +325,12 @@ sub create_new_order {
     );
 }
 
-=head2 I<OBJ>->get_authorization( URL )
+=head2 $authz = I<OBJ>->get_authorization( $URL )
 
-Fetches the authorization’s information based on the given URL
+Fetches the authorization’s information based on the given $URL
 and returns a L<Net::ACME2::Authorization> object.
 
-The URL is as given by L<Net::ACME2::Order>.
+The URL is as given by L<Net::ACME2::Order>’s C<authorizations()> method.
 
 =cut
 
@@ -350,11 +374,11 @@ sub get_authorization {
 #    );
 #}
 
-=head2 I<OBJ>->make_key_authorization( CHALLENGE )
+=head2 $str = I<OBJ>->make_key_authorization( $CHALLENGE )
 
 Accepts an instance of L<Net::ACME2::Challenge> (probably a subclass
 thereof) and returns
-a key authorization string suitable for handling the given CHALLENGE.
+a key authorization string suitable for handling the given $CHALLENGE.
 See F</examples> in the distribution for example usage.
 
 If you’re using HTTP authorization and are on the same server as the
@@ -367,7 +391,7 @@ handle HTTP challenges.
 sub make_key_authorization {
     my ($self, $challenge_obj) = @_;
 
-    die 'Need a challenge object!' if !$challenge_obj;
+    _die_generic('Need a challenge object!') if !$challenge_obj;
 
     return $challenge_obj->token() . '.' . $self->_key_thumbprint();
 }
@@ -381,7 +405,7 @@ Signal to the ACME server that the CHALLENGE is ready.
 sub accept_challenge {
     my ($self, $challenge_obj) = @_;
 
-    my $post = $self->_post_url(
+    $self->_post_url(
         $challenge_obj->url(),
         {
             keyAuthorization => $self->make_key_authorization($challenge_obj),
@@ -391,24 +415,27 @@ sub accept_challenge {
     return;
 }
 
-=head2 I<OBJ>->poll_authorization( AUTHORIZATION )
+=head2 $status = I<OBJ>->poll_authorization( $AUTHORIZATION )
 
 Accepts a L<Net::ACME2::Authorization> instance and polls the
 ACME server for that authorization’s status. The AUTHORIZATION
 object is then updated with the results of the poll.
 
+As a courtesy, this returns the object’s new C<status()>.
+
 =cut
 
 #This has to handle updates to the authz and challenge objects
 *poll_authorization = *_poll_order_or_authz;
 
-=head2 I<OBJ>->finalize_order( ORDER, CSR )
+=head2 $status = I<OBJ>->finalize_order( $ORDER, $CSR )
 
-Finalizes an order and updates the ORDER object with the returned
-status. The CSR may be in either DER or PEM format.
+Finalizes an order and updates the $ORDER object with the returned
+status. $CSR may be in either DER or PEM format.
 
-ORDER may have C<status()> of C<valid> after this operation,
-or you may need to C<poll_order()>.
+As a courtesy, this returns the $ORDER’s C<status()>. If this does
+not equal C<valid>, then you should probably C<poll_order()>
+until it does.
 
 =cut
 
@@ -436,10 +463,10 @@ sub finalize_order {
 
     $order_obj->update($content);
 
-    return;
+    return $order_obj->status();
 }
 
-=head2 I<OBJ>->poll_order( ORDER )
+=head2 $status = I<OBJ>->poll_order( $ORDER )
 
 Like C<poll_authorization()> but handles a
 L<Net::ACME2::Order> object instead.
@@ -464,7 +491,11 @@ sub _get_directory {
         $self->{'_ua'}->get("https://$self->{'_host'}$dir_path")->content_struct();
     };
 
-    $self->{'_ua'}->set_new_nonce_url( $self->{'_directory'}{'newNonce'} );
+    my $new_nonce_url = $self->{'_directory'}{'newNonce'} or do {
+        _die_generic('Directory is missing “newNonce”.');
+    };
+
+    $self->{'_ua'}->set_new_nonce_url( $new_nonce_url );
 
     return $self->{'_directory'};
 }
@@ -473,7 +504,7 @@ sub _require_key_id {
     my ($self, $opts_hr) = @_;
 
     $opts_hr->{'_key_id'} = $self->{'_key_id'} or do {
-        die 'No key ID has been set. Either pass “key_id” to new(), or create_new_account().';
+        _die_generic('No key ID has been set. Either pass “key_id” to new(), or create_new_account().');
     };
 
     return
@@ -488,13 +519,7 @@ sub _poll_order_or_authz {
 
     $order_or_authz_obj->update($content);
 
-    my $status = $order_or_authz_obj->status();
-
-    return 1 if $status eq 'valid';
-    return 0 if $status eq 'pending' || $status eq 'processing';
-
-    use Data::Dumper;
-    die Dumper $order_or_authz_obj; #TODO
+    return $order_or_authz_obj->status();
 }
 
 sub _key_obj {
@@ -509,7 +534,6 @@ sub _set_ua {
     $self->{'_ua'} = Net::ACME2::HTTP->new(
         key => $self->_key_obj(),
         key_id => $self->{'_key_id'},
-        jws_format => $self->JWS_FORMAT(),
     );
 
     return;
@@ -523,7 +547,9 @@ sub _post {
     my $post_method;
     $post_method = 'post_full_jwt' if grep { $link_name eq $_ } FULL_JWT_METHODS();
 
-    my $url = $self->_get_directory()->{$link_name} or die "Unknown link name: “$link_name”";
+    # Since the $link_name will come from elsewhere in this module
+    # there really shouldn’t be an error here, but just in case.
+    my $url = $self->_get_directory()->{$link_name} or _die_generic("Unknown link name: “$link_name”");
 
     return $self->_post_url( $url, $data, $post_method );
 }
@@ -540,6 +566,10 @@ sub _post_url {
     return $self->{'_ua'}->$post_method( $url, $data );
 }
 
+sub _die_generic {
+    die Net::ACME2::X->create('Generic', @_);
+}
+
 1;
 
 =head1 TODO
@@ -549,7 +579,7 @@ sub _post_url {
 =item * Add pre-authorization support if there is ever a production
 use for it.
 
-=item * Tighten up challenge failure response.
+=item * Expose the Retry-After header via the module API.
 
 =item * Add (more) tests.
 
